@@ -1,7 +1,7 @@
 /*
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
- * Copyright (C) 2017-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2017-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,7 +57,7 @@ typedef struct _RemminaAppletMenuItem {
 	gchar *				group;
 	gchar *				protocol;
 	gchar *				server;
-	gboolean			ssh_enabled;
+	gboolean			ssh_tunnel_enabled;
 } RemminaAppletMenuItem;
 
 typedef struct _RemminaAppletMenuItemClass {
