@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include <string.h>
 
 #include "remmina_public.h"
+#include "remmina_pref.h"
 #include "remmina_string_array.h"
 #include "remmina_plugin_manager.h"
 #include "remmina_file_manager.h"
@@ -55,27 +56,33 @@ gchar *remmina_file_get_datadir(void)
 	TRACE_CALL(__func__);
 	const gchar *dir = ".remmina";
 	int i;
+	/* From preferences, datadir_path */
+	remminadir = remmina_pref_get_value("datadir_path");
+	if (remminadir != NULL && strlen(remminadir) > 0)
+		if (g_file_test(remminadir, G_FILE_TEST_IS_DIR))
+			return remminadir;
+	g_free(remminadir), remminadir = NULL;
 	/* Legacy ~/.remmina */
 	remminadir = g_build_path("/", g_get_home_dir(), dir, NULL);
 	if (g_file_test(remminadir, G_FILE_TEST_IS_DIR))
 		return remminadir;
 	g_free(remminadir), remminadir = NULL;
 	/* ~/.local/share/remmina */
-	remminadir = g_build_path( "/", g_get_user_data_dir(), "remmina", NULL);
+	remminadir = g_build_path("/", g_get_user_data_dir(), "remmina", NULL);
 	if (g_file_test(remminadir, G_FILE_TEST_IS_DIR))
 		return remminadir;
 	g_free(remminadir), remminadir = NULL;
 	/* /usr/local/share/remmina */
-	const gchar * const *dirs = g_get_system_data_dirs();
+	const gchar *const *dirs = g_get_system_data_dirs();
 	g_free(remminadir), remminadir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
-		remminadir = g_build_path( "/", dirs[i], "remmina", NULL);
+		remminadir = g_build_path("/", dirs[i], "remmina", NULL);
 		if (g_file_test(remminadir, G_FILE_TEST_IS_DIR))
 			return remminadir;
 		g_free(remminadir), remminadir = NULL;
 	}
 	/* The last case we use  the home ~/.local/share/remmina */
-	remminadir = g_build_path( "/", g_get_user_data_dir(), "remmina", NULL);
+	remminadir = g_build_path("/", g_get_user_data_dir(), "remmina", NULL);
 	return remminadir;
 }
 
@@ -100,12 +107,12 @@ void remmina_file_manager_init(void)
 	const gchar *filename;
 	int i;
 
-	remminadir = g_build_path( "/", g_get_user_data_dir(), "remmina", NULL);
+	remminadir = g_build_path("/", g_get_user_data_dir(), "remmina", NULL);
 	/* Create the XDG_USER_DATA directory */
 	g_mkdir_with_parents(remminadir, 0750);
 	g_free(remminadir), remminadir = NULL;
 	/* Create the XDG_CACHE_HOME directory */
-	cachedir = g_build_path( "/", g_get_user_cache_dir(), "remmina", NULL);
+	cachedir = g_build_path("/", g_get_user_cache_dir(), "remmina", NULL);
 	g_mkdir_with_parents(cachedir, 0750);
 	g_free(cachedir), cachedir = NULL;
 	/* Empty legacy ~/.remmina */
@@ -114,24 +121,24 @@ void remmina_file_manager_init(void)
 		dir = g_dir_open(remminadir, 0, NULL);
 		while ((filename = g_dir_read_name(dir)) != NULL) {
 			remmina_file_manager_do_copy(
-				g_build_path( "/", remminadir, filename, NULL),
-				g_build_path( "/", g_get_user_data_dir(),
-					"remmina", filename, NULL));
+				g_build_path("/", remminadir, filename, NULL),
+				g_build_path("/", g_get_user_data_dir(),
+					     "remmina", filename, NULL));
 		}
 	}
 
 	/* XDG_DATA_DIRS, i.e. /usr/local/share/remmina */
-	const gchar * const *dirs = g_get_system_data_dirs();
+	const gchar *const *dirs = g_get_system_data_dirs();
 	g_free(remminadir), remminadir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
-		remminadir = g_build_path( "/", dirs[i], "remmina", NULL);
+		remminadir = g_build_path("/", dirs[i], "remmina", NULL);
 		if (g_file_test(remminadir, G_FILE_TEST_IS_DIR)) {
 			dir = g_dir_open(remminadir, 0, NULL);
 			while ((filename = g_dir_read_name(dir)) != NULL) {
 				remmina_file_manager_do_copy(
-					g_build_path( "/", remminadir, filename, NULL),
-					g_build_path( "/", g_get_user_data_dir(),
-						"remmina", filename, NULL));
+					g_build_path("/", remminadir, filename, NULL),
+					g_build_path("/", g_get_user_data_dir(),
+						     "remmina", filename, NULL));
 			}
 		}
 		g_free(remminadir), remminadir = NULL;
@@ -145,11 +152,11 @@ gint remmina_file_manager_iterate(GFunc func, gpointer user_data)
 {
 	TRACE_CALL(__func__);
 	gchar filename[MAX_PATH_LEN];
-	GDir* dir;
-	const gchar* name;
-	RemminaFile* remminafile;
+	GDir *dir;
+	const gchar *name;
+	RemminaFile *remminafile;
 	gint items_count = 0;
-	gchar* remmina_data_dir;
+	gchar *remmina_data_dir;
 
 	remmina_data_dir = remmina_file_get_datadir();
 	dir = g_dir_open(remmina_data_dir, 0, NULL);
@@ -159,7 +166,7 @@ gint remmina_file_manager_iterate(GFunc func, gpointer user_data)
 			if (!g_str_has_suffix(name, ".remmina"))
 				continue;
 			g_snprintf(filename, MAX_PATH_LEN, "%s/%s",
-				remmina_data_dir, name);
+				   remmina_data_dir, name);
 			remminafile = remmina_file_load(filename);
 			if (remminafile) {
 				(*func)(remminafile, user_data);
@@ -173,17 +180,17 @@ gint remmina_file_manager_iterate(GFunc func, gpointer user_data)
 	return items_count;
 }
 
-gchar* remmina_file_manager_get_groups(void)
+gchar *remmina_file_manager_get_groups(void)
 {
 	TRACE_CALL(__func__);
 	gchar filename[MAX_PATH_LEN];
-	GDir* dir;
-	const gchar* name;
-	RemminaFile* remminafile;
-	RemminaStringArray* array;
-	const gchar* group;
-	gchar* groups;
-	gchar* remmina_data_dir;
+	GDir *dir;
+	const gchar *name;
+	RemminaFile *remminafile;
+	RemminaStringArray *array;
+	const gchar *group;
+	gchar *groups;
+	gchar *remmina_data_dir;
 
 	remmina_data_dir = remmina_file_get_datadir();
 	array = remmina_string_array_new();
@@ -199,9 +206,8 @@ gchar* remmina_file_manager_get_groups(void)
 		remminafile = remmina_file_load(filename);
 		if (remminafile) {
 			group = remmina_file_get_string(remminafile, "group");
-			if (group && remmina_string_array_find(array, group) < 0) {
+			if (group && remmina_string_array_find(array, group) < 0)
 				remmina_string_array_add(array, group);
-			}
 			remmina_file_free(remminafile);
 		}
 	}
@@ -213,15 +219,15 @@ gchar* remmina_file_manager_get_groups(void)
 	return groups;
 }
 
-static void remmina_file_manager_add_group(GNode* node, const gchar* group)
+static void remmina_file_manager_add_group(GNode *node, const gchar *group)
 {
 	TRACE_CALL(__func__);
 	gint cmp;
-	gchar* p1;
-	gchar* p2;
-	GNode* child;
+	gchar *p1;
+	gchar *p2;
+	GNode *child;
 	gboolean found;
-	RemminaGroupData* data;
+	RemminaGroupData *data;
 
 	if (node == NULL)
 		return;
@@ -238,7 +244,7 @@ static void remmina_file_manager_add_group(GNode* node, const gchar* group)
 	found = FALSE;
 
 	for (child = g_node_first_child(node); child; child = g_node_next_sibling(child)) {
-		cmp = g_strcmp0(((RemminaGroupData*)child->data)->name, p1);
+		cmp = g_strcmp0(((RemminaGroupData *)child->data)->name, p1);
 
 		if (cmp == 0) {
 			found = TRUE;
@@ -252,33 +258,30 @@ static void remmina_file_manager_add_group(GNode* node, const gchar* group)
 	if (!found) {
 		data = g_new0(RemminaGroupData, 1);
 		data->name = p1;
-		if (node->data) {
-			data->group = g_strdup_printf("%s/%s", ((RemminaGroupData*)node->data)->group, p1);
-		}else  {
+		if (node->data)
+			data->group = g_strdup_printf("%s/%s", ((RemminaGroupData *)node->data)->group, p1);
+		else
 			data->group = g_strdup(p1);
-		}
-		if (child) {
+		if (child)
 			child = g_node_insert_data_before(node, child, data);
-		}else  {
+		else
 			child = g_node_append_data(node, data);
-		}
 	}
 	remmina_file_manager_add_group(child, p2);
 
-	if (found) {
+	if (found)
 		g_free(p1);
-	}
 }
 
-GNode* remmina_file_manager_get_group_tree(void)
+GNode *remmina_file_manager_get_group_tree(void)
 {
 	TRACE_CALL(__func__);
 	gchar filename[MAX_PATH_LEN];
-	GDir* dir;
-	const gchar* name;
-	RemminaFile* remminafile;
-	const gchar* group;
-	GNode* root;
+	GDir *dir;
+	const gchar *name;
+	RemminaFile *remminafile;
+	const gchar *group;
+	GNode *root;
 
 	root = g_node_new(NULL);
 
@@ -301,42 +304,39 @@ GNode* remmina_file_manager_get_group_tree(void)
 	return root;
 }
 
-void remmina_file_manager_free_group_tree(GNode* node)
+void remmina_file_manager_free_group_tree(GNode *node)
 {
 	TRACE_CALL(__func__);
-	RemminaGroupData* data;
-	GNode* child;
+	RemminaGroupData *data;
+	GNode *child;
 
 	if (!node)
 		return;
-	data = (RemminaGroupData*)node->data;
+	data = (RemminaGroupData *)node->data;
 	if (data) {
 		g_free(data->name);
 		g_free(data->group);
 		g_free(data);
 		node->data = NULL;
 	}
-	for (child = g_node_first_child(node); child; child = g_node_next_sibling(child)) {
+	for (child = g_node_first_child(node); child; child = g_node_next_sibling(child))
 		remmina_file_manager_free_group_tree(child);
-	}
 	g_node_unlink(node);
 }
 
-RemminaFile* remmina_file_manager_load_file(const gchar* filename)
+RemminaFile *remmina_file_manager_load_file(const gchar *filename)
 {
 	TRACE_CALL(__func__);
-	RemminaFile* remminafile = NULL;
-	RemminaFilePlugin* plugin;
-	gchar* p;
+	RemminaFile *remminafile = NULL;
+	RemminaFilePlugin *plugin;
+	gchar *p;
 
 	if ((p = strrchr(filename, '.')) != NULL && g_strcmp0(p + 1, "remmina") == 0) {
 		remminafile = remmina_file_load(filename);
-	}else  {
+	} else {
 		plugin = remmina_plugin_manager_get_import_file_handler(filename);
-		if (plugin) {
+		if (plugin)
 			remminafile = plugin->import_func(filename);
-		}
 	}
 	return remminafile;
 }
-
