@@ -1,6 +1,6 @@
 /*
  * Remmina - The GTK+ Remote Desktop Client
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -159,8 +159,6 @@
 #endif
 #include "remmina_stats.h"
 
-struct utsname u;
-
 struct ProfilesData {
 	GHashTable *proto_count;
 	GHashTable *proto_date;
@@ -864,9 +862,6 @@ JsonNode *remmina_stats_get_all()
 	json_builder_set_member_name(b, "REMMINAVERSION");
 	json_builder_add_value(b, n);
 
-	if (uname(&u) == -1)
-		g_print("uname:");
-
 	n = remmina_stats_get_os_info();
 	json_builder_set_member_name(b, "SYSTEM");
 	json_builder_add_value(b, n);
